@@ -1,6 +1,7 @@
 """Configure pyramid_openapi3 addon."""
 
 from .exceptions import extract_errors
+from .exceptions import MissingEndpointsError
 from .exceptions import RequestValidationError
 from .exceptions import ResponseValidationError
 from .wrappers import PyramidOpenAPIRequestFactory
@@ -10,9 +11,11 @@ from openapi_core.validation.request.validators import RequestValidator
 from openapi_core.validation.response.validators import ResponseValidator
 from openapi_spec_validator import validate_spec
 from openapi_spec_validator.schemas import read_yaml_file
+from pathlib import Path
 from pyramid.config import Configurator
 from pyramid.config import PHASE0_CONFIG
 from pyramid.config.views import ViewDeriverInfo
+from pyramid.events import ApplicationCreated
 from pyramid.exceptions import ConfigurationError
 from pyramid.httpexceptions import exception_response
 from pyramid.path import AssetResolver
@@ -36,7 +39,9 @@ def includeme(config: Configurator) -> None:
     config.add_directive("pyramid_openapi3_add_formatter", add_formatter)
     config.add_directive("pyramid_openapi3_add_explorer", add_explorer_view)
     config.add_directive("pyramid_openapi3_spec", add_spec_view)
+    config.add_directive("pyramid_openapi3_spec_directory", add_spec_view_directory)
     config.add_tween("pyramid_openapi3.tween.response_tween_factory", over=EXCVIEW)
+    config.add_subscriber(check_all_routes, ApplicationCreated)
 
     if not config.registry.settings.get(  # pragma: no branch
         "pyramid_openapi3_extract_errors"
@@ -170,6 +175,13 @@ def add_spec_view(
     """
 
     def register() -> None:
+        settings = config.registry.settings.get("pyramid_openapi3")
+        if settings and settings.get("spec") is not None:
+            raise ConfigurationError(
+                "Spec has already been configured. You may only call "
+                "pyramid_openapi3_spec or pyramid_openapi3_spec_directory once"
+            )
+
         if hupper.is_active():  # pragma: no cover
             hupper.get_reloader().watch_files([filepath])
         spec_dict = read_yaml_file(filepath)
@@ -200,6 +212,60 @@ def add_spec_view(
     config.action(("pyramid_openapi3_spec",), register, order=PHASE0_CONFIG)
 
 
+def add_spec_view_directory(
+    config: Configurator,
+    filepath: str,
+    route: str = "/spec",
+    route_name: str = "pyramid_openapi3.spec",
+) -> None:
+    """Serve and register OpenApi 3.0 specification directory.
+
+    :param filepath: absolute/relative path to the root specification file
+    :param route: URL path where to serve specification file
+    :param route_name: Route name under which specification file will be served
+    """
+
+    def register() -> None:
+        settings = config.registry.settings.get("pyramid_openapi3")
+        if settings and settings.get("spec") is not None:
+            raise ConfigurationError(
+                "Spec has already been configured. You may only call "
+                "pyramid_openapi3_spec or pyramid_openapi3_spec_directory once"
+            )
+        if route.endswith((".yaml", ".yml", ".json")):
+            raise ConfigurationError(
+                "Having route be a filename is not allowed when using a spec directory"
+            )
+
+        path = Path(filepath).resolve()
+        if hupper.is_active():  # pragma: no cover
+            hupper.get_reloader().watch_files(list(path.parent.iterdir()))
+
+        spec_dict = read_yaml_file(path)
+        spec_url = path.as_uri()
+        validate_spec(spec_dict, spec_url=spec_url)
+        spec = create_spec(spec_dict, spec_url=spec_url)
+
+        config.add_static_view(route, str(path.parent))
+        config.add_route(route_name, f"{route}/{path.name}")
+
+        custom_formatters = config.registry.settings.get("pyramid_openapi3_formatters")
+
+        config.registry.settings["pyramid_openapi3"] = {
+            "filepath": filepath,
+            "spec_route_name": route_name,
+            "spec": spec,
+            "request_validator": RequestValidator(
+                spec, custom_formatters=custom_formatters
+            ),
+            "response_validator": ResponseValidator(
+                spec, custom_formatters=custom_formatters
+            ),
+        }
+
+    config.action(("pyramid_openapi3_spec",), register, order=PHASE0_CONFIG)
+
+
 def openapi_validation_error(
     context: t.Union[RequestValidationError, ResponseValidationError], request: Request
 ) -> Response:
@@ -224,3 +290,31 @@ def openapi_validation_error(
         status_code = 500
 
     return exception_response(status_code, json_body=errors)
+
+
+def check_all_routes(event: ApplicationCreated):
+    """Assert all endpoints in the spec are registered as routes.
+
+    Listen for ApplicationCreated event and assert all endpoints defined in
+    the API spec have been registered as Pyramid routes.
+    """
+
+    app = event.app
+    openapi_settings = app.registry.settings.get("pyramid_openapi3")
+    if not openapi_settings:
+        # pyramid_openapi3 not configured?
+        logger.warning(
+            "pyramid_openapi3 settings not found. "
+            "Did you forget to call config.pyramid_openapi3_spec?"
+        )
+        return
+
+    if not openapi_settings.get("enable_endpoint_validation", True):
+        logger.info("Endpoint validation against specification is disabled")
+        return
+    paths = list(openapi_settings["spec"].paths.keys())
+    routes = [route.path for name, route in app.routes_mapper.routes.items()]
+
+    missing = [r for r in paths if r not in routes]
+    if missing:
+        raise MissingEndpointsError(missing)
